<template>
  <div ref="chart" class="chart">
    <svg ref="canvas" id="target">
    </svg>
  </div>
</template>

<script setup>

import * as d3 from 'd3'

const chart = ref()
const canvas = ref()

const data = d3.csvParse(`letter,frequency
A,0.08167
B,0.01492
C,0.02782
D,0.04253
E,0.12702
F,0.02288
G,0.02015
H,0.06094
I,0.06966
J,0.00153
K,0.00772
L,0.04025
M,0.02406
N,0.06749
O,0.07507
P,0.01929
Q,0.00095
R,0.05987
S,0.06327
T,0.09056
U,0.02758
V,0.00978
W,0.0236
X,0.0015
Y,0.01974
Z,0.00074
`)

onMounted(()=>{

  // Specify the chart’s dimensions.
  const width = 928;
  const height = 500;
  const marginTop = 20;
  const marginRight = 0;
  const marginBottom = 30;
  const marginLeft = 40;
  const borderRadius = 1;

  const mainColor = "#52824e"
  const bgColor = "#344224"
  const highlightColor = "#6C9568"

  const t = d3.transition()
    .duration(1000)
    .ease(d3.easeExpInOut);

  // Create the horizontal scale and its axis generator.
  const x = d3.scaleBand()
    .domain(d3.sort(data, d => -d.frequency).map(d => d.letter))
    .range([marginLeft, width - marginRight])
    .padding(0.1);

  const xAxis = d3.axisBottom(x).tickSizeOuter(0);

  // Create the vertical scale.
  const y = d3.scaleLinear()
    .domain([0, d3.max(data, d => d.frequency)]).nice()
    .range([height - marginBottom, marginTop]);

  const yAxis = d3.axisLeft(y).tickSizeOuter(0);

  // Create the SVG container and call the zoom behavior.
  const svg = d3.select(canvas.value)
    .attr("viewBox", [0, 0, width, height])
    .attr("width", width)
    .attr("height", height)
    .attr("style", "max-width: 100%; height: auto;")
    .call(zoom)
    .on("pointerenter pointermove", pointermoved)
    .on("pointerleave", pointerleft)
    .on("touchstart", event => event.preventDefault());

  // Append the axes.
  svg.append("g")
    .attr("class", "x-axis")
    .attr("transform", `translate(0,${height - marginBottom})`)
    .call(xAxis);

  svg.append("g")
    .attr("class", "y-axis")
    .attr("transform", `translate(${marginLeft},0)`)
    .call(yAxis)
    .call(g => g.select(".domain").remove());

  const numberOfTicks = 5;

  var yAxisGrid = yAxis.ticks(numberOfTicks)
    .tickSize(-width, 0)
    .tickFormat("")

  svg.append("g")
    .attr("transform", `translate(${marginLeft},0)`)
    .classed('y', true)
    .classed('grid', true)
    .call(yAxisGrid)
    .selectAll(".tick")
    .style("color", "lightgray")

  // Append the bars.
  svg.append("g")
    .attr("class", "bars")
    .attr("fill", mainColor)
    .selectAll("rect")
    .data(data)
    .join("rect")
    .attr("x", d => x(d.letter))
    .attr("y", height - marginBottom + borderRadius)
    .attr("letter", d => d.letter)
    .attr("rx", borderRadius)
    .attr("height", 0)
    .attr("width", x.bandwidth())

  d3.select(".bars")
    .selectAll("rect")
    .data(data)
    .transition(t)
    .attr("height", d => y(0) - y(d.frequency) + borderRadius)
    .attr("y", d => y(d.frequency))

  function zoom(svg) {
    const extent = [[marginLeft, marginTop], [width - marginRight, height - marginTop]];

    svg.call(d3.zoom()
        .scaleExtent([1, 8])
        .translateExtent(extent)
        .extent(extent)
        .on("zoom", zoomed));

    function zoomed(event) {
      x.range([marginLeft, width - marginRight].map(d => event.transform.applyX(d)));
      svg.selectAll(".bars rect").attr("x", d => x(d.letter)).attr("width", x.bandwidth());
      svg.selectAll(".x-axis").call(xAxis);
    }
  }

  // Create the tooltip container.
  const tooltip = d3.select(chart.value).append('div')

  function formatValue(value) {
    return Number(value).toLocaleString("fr-CA");
  }
  
  // Add the event listeners that show or hide the tooltip.
  const bisect = d3.bisector(d => d.letter).center;
  const tt = () => transition()
    .duration(100)
    .ease(easeExpInOut)

  function pointermoved(event) {
    if(event.target.tagName == 'rect'){
      const i = bisect(data, event.target.attributes.letter.value);

      tooltip.style("display", null);
      tooltip.style("transform", `translate(calc(-20px + ${x(data[i].letter) + x.bandwidth()}px),calc(${y(data[i].frequency)}px + 20px))`);
      tooltip.classed('tooltip', true)
      // tooltip.style("padding", 20);
      // tooltip.style("position", 'absolute');

      // d3.selectAll("rect").transition(tt).style("fill", mainColor)
      // d3.select(event.target).transition(tt).style("fill", "darkblue")
  
      const text = tooltip.selectAll("div")
        .data([data[i].letter, formatValue(data[i].frequency)])
        .join("div")
        .text(d => d);
        // .attr("fill", "white")
        // .attr("x", 0)
        // .attr("y", (_, i) => `${i * 1.1}em`)
        // .attr("font-weight", (_, i) => i ? null : "bold")
  
    }
  }

  // Reset tooltip location
  function pointerleft() {
    tooltip.style("display", "none");
  }

  // Hover effects
  d3.selectAll("rect").on("mouseover", function (e) {
    d3.select(e.target).transition(tt).style("fill", highlightColor);
  }).on("mouseout", function (e) {
    d3.select(e.target).transition(tt).style("fill", mainColor);
  })

})

</script>

<style lang="scss" scoped>

.chart{
  position: relative;
}

:deep() .tooltip{
  pointer-events: none;
  box-sizing: border-box;
  --tooltip-bg-color: #344224;
  --tooltip-color: white;
  top: 0;
  left: 0;
  position: absolute;
  padding: 20px 20px;
  background-color: var(--tooltip-bg-color);
  color: var(--tooltip-color);
  font-family: var(--tooltip-font-family);
  border-radius: 10px;
  display: flex;
  flex-direction: column;
  gap: 10px;
  div:first-child{
    font-weight: 700;
  }
}

</style>